<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * HTTP helper for call external service
 * @package     quizaccess_quilgo
 * @copyright   2023 Native Platform Ltd <hello@quilgo.com>
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace quizaccess_quilgo;

class http {
    /**
     * @var mixed curl instance
     */
    protected $ch;

    /**
     * @var array used headers
     */
    protected $headers = [];

    /**
     * @var array used payload
     */
    protected $payload = [];

    /**
     * @var string used url
     */
    protected $url;

    /**
     * Constructor class
     * @param string $url
     * @param array $payload
     * @return void
     */
    public function __construct($url = null, $payload = []) {
        $this->ch = curl_init();
        $this->url = $url;
        $this->payload = $payload;

        curl_setopt($this->ch, CURLOPT_URL, $this->url);
        curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($this->ch, CURLOPT_HEADER, 1);
    }

    /**
     * Init class
     * @param string $url
     * @param array $payload
     * @return void
     */
    public function factory($url = null, $payload = []) {
        return new self($url, $payload);
    }

    /**
     * Init class
     * @param string $str
     * @return string
     */
    protected static function fix_string_case($str) {
        $str = explode('-', $str);
        foreach ($str as &$word) {
            $word = ucfirst($word);
        }
        return implode('-', $str);
    }

    /**
     * Set headers
     * @param mixed $data
     * @return $this
     */
    public function set_headers($data) {
        foreach ($data as $key => $val) {
            $this->headers[self::fix_string_case($key)] = $val;
        }
        return $this;
    }

    /**
     * Set response after request call
     * @return mixed
     */
    protected function response() {
        $response = curl_exec($this->ch);
        $headersize = curl_getinfo($this->ch, CURLINFO_HEADER_SIZE);
        $status = curl_getinfo($this->ch, CURLINFO_HTTP_CODE);
        $error = curl_error($this->ch);
        $errno = curl_errno($this->ch);

        curl_close($this->ch);
        if ($response === false) {
            throw new \RuntimeException($error, $errno);
        }

        $content = substr($response, $headersize);

        $response = new \stdClass();
        $response->status = $status;
        $response->body = json_decode($content);

        return $response;
    }

    /**
     * Handle post request
     * @return mixed
     */
    public function post() {
        $this->payload = json_encode($this->payload);
        $this->headers = array_merge($this->headers, ['Content-Length: ' . strlen($this->payload)]);

        curl_setopt($this->ch, CURLOPT_POST, 1);
        curl_setopt($this->ch, CURLOPT_POSTFIELDS, $this->payload);
        curl_setopt($this->ch, CURLOPT_HTTPHEADER, $this->headers);

        return $this->response();
    }

    /**
     * Handle get request
     * @return mixed
     */
    public function get() {
        curl_setopt($this->ch, CURLOPT_HTTPHEADER, $this->headers);

        return $this->response();
    }
}
